# Generated by Django 3.2.13 on 2023-03-30 07:14

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=300)),
                ('last_name', models.CharField(max_length=300)),
                ('email', models.EmailField(max_length=254)),
                ('image', models.ImageField(blank=True, null=True, upload_to='dsmauth-image')),
                ('role', models.JSONField()),
                ('permission', models.JSONField()),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.urls import path
from rest_framework_simplejwt.views import TokenObtainPairView,TokenRefreshView
from .views import logout

urlpatterns = [
    path('token/', TokenObtainPairView.as_view(), name='token_obtain_pair'),
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),
    path('logout/', logout, name='logout')
]



from social_core.backends.oauth import BaseOAuth2
import requests
from django.conf import settings
from urllib.parse import urlencode, unquote

class dsmOAuth2(BaseOAuth2):
    name = 'dsmauth'
    AUTH_SERVER = settings.OAUTH_DSM_SERVER_BASEURL
    _base = f'http://{AUTH_SERVER}'
    if settings.OAUTH_DSM_SCHEME == "https":
        _base = f'https://{AUTH_SERVER}'
    AUTHORIZATION_URL = f'{_base}/o/authorize'
    ACCESS_TOKEN_URL = f'{_base}/o/token/'
    REFRESH_TOKEN_URL = f'{_base}/o/token/'
    USER_DATA_URL = f'{_base}/api/v1/account/me'
    
    _internal_host = getattr(settings, 'OAUTH_INTERNAL_IP', None)
    if _internal_host not in [None, "", " "]:
        ACCESS_TOKEN_URL = f'http://{_internal_host}/o/token/'
        REFRESH_TOKEN_URL = f'http://{_internal_host}/o/token/'
        USER_DATA_URL = f'http://{_internal_host}/api/v1/account/me'
        
    ACCESS_TOKEN_METHOD = 'POST'
    REVOKE_TOKEN_METHOD = 'GET'
    

    SCOPE_SEPARATOR = ' '
    EXTRA_DATA = [
        ('expires_in', 'expires_in'),
        ('refresh_token', 'refresh_token'),
        ('scope', 'scope'),
    ]

    def get_user_id(self, details, response):
        return details['username']

    def get_user_details(self, response):
        res = {
            'username': response.get('username'),
            'email': response.get('email'),
            'first_name': response.get('first_name'),
            'last_name': response.get('last_name'),
        }
        return res

    def user_data(self, access_token, *args, **kwargs):
        data = self._user_data(access_token)
        return data
                        
    def _user_data(self, access_token, path=None):
        headers = {
            'Authorization': 'Bearer {0}'.format(access_token)
        }

        extra_data = requests.get(self.USER_DATA_URL, headers=headers)

        user_profile = extra_data.json()

        return user_profile

    def auth_url(self):
        url = super().auth_url()
        callback = self.data.get('callback', None)
        next = self.data.get('next', None)
        if callback is not None:
            url += f"&callback={callback}"
        if next is not None:
            url += f"&next={next}"
        return url


from django.db import models
from django.contrib.auth.models import User, AnonymousUser

class Account(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    first_name = models.CharField(max_length=300)
    last_name = models.CharField(max_length=300)
    email = models.EmailField()
    image = models.ImageField(upload_to='dsmauth-image', blank=True, null=True)
    role = models.JSONField()
    permission = models.JSONField()
    
    def __str__(self) -> str:
        return f"{self.user.username}"

import requests
from django.conf import settings
from django.contrib.auth.signals import user_logged_in
from PIL import Image
from io import BytesIO
from django.core.files.images import ImageFile

def logged_in_handle(sender, user, request, **kwargs):
    if settings.DEBUG: print("logged_in_handle")
    prov = user.social_auth.filter(provider='dsmauth')
    user_data = {
        'user': user,
        'first_name': user.first_name,
        'last_name': user.last_name,
        'email': user.email,
        'role': [],
        'permission': []
    }
    if prov.exists():
        data = prov.last().extra_data
        headers = {
            "Authorization": f"Bearer {data.get('access_token', '')}"
        }
        _internal_ip = getattr(settings, 'OAUTH_INTERNAL_IP', None)
        _url = f"http://{_internal_ip}" if _internal_ip not in [None, "", " "] else  f"https://{settings.OAUTH_DSM_SERVER_BASEURL}"
        try:
            api = requests.get(f"{_url}/api/v1/account/me/", headers=headers).json()
        except Exception as e:
            if settings.DEBUG: print(f"logged_in_handle: {e}")
            pass
        else:
            account = Account.objects.filter(user=request.user) | Account.objects.filter(user__username=api.get('username', ''))
            user_data.update({
                'image': ImageFile(BytesIO(requests.get(api.get('profile_url')).content), name=f"{user.username}.png") if api.get('profile_url', "") != "" else None,
                'role': api.get('role', []),
                'permission': api.get('permission', [])
            })
            user = request.user
            user.is_staff = api.get('is_staff', False)
            user.is_superuser = api.get('is_superuser', False)
            user.save()
            if not account.exists(): 
                Account.objects.create(**user_data)
            else:
                account.update(**user_data)
    elif type(request.user) != AnonymousUser and Account.objects.filter(user__username=user.username).exists() == False:
        Account.objects.create(**user_data)
        
user_logged_in.connect(logged_in_handle)


import traceback
import re
from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from django.http.response import HttpResponse
from rest_framework_simplejwt.authentication import JWTAuthentication

class JWTauthenticationMiddleware(MiddlewareMixin):
    def process_request(self, request):
        _user = JWTAuthentication().authenticate(request)
        request.user = _user[0] if _user != None else request.user

class LogHeaderMiddleware(MiddlewareMixin):
    
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        if request.user.is_authenticated:
            response['X-Username'] = request.user.username
        else:
            response['X-Username'] = 'Anonymous'
        return response
    
    def process_exception(self, request, exception):
        if not settings.DEBUG:
            if exception:
                message = "**{url}**\n\n{error}\n\n````{tb}````".format(
                    url=request.build_absolute_uri(),
                    error=repr(exception),
                    tb=traceback.format_exc()
                )
                message = re.sub("\n", "\t", message)
                response = HttpResponse(
                    "<h1>Internal server error (500)</h1><p>error save to log</p>", 
                    status=500, 
                    headers={
                        'X-Error': message
                    }
                )
            return response


from django.contrib import admin
from . import models

class AccountAdmin(admin.ModelAdmin):
    list_display = [f.name for f in models.Account._meta.fields]
admin.site.register(models.Account, AccountAdmin)